﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.9
@EndOfDesignText@
#Event: CheckProgress (EventArgs As CheckPasswordEventArgs)
#Event: CheckDone (EventArgs As CheckPasswordEventArgs)

Private Sub Class_Globals
	Private internalCharSet As String = "ISO-8859-1" 'ANSI charset
	Private internalDatabaseDirName As String
	Private internalDatabaseFileName As String
	Private internalEventName As String = ""
	Private internalOwner As Object = Null
End Sub

Public Sub BytesToHex (Buffer() As Byte) As String
	If Buffer.Length Mod 4 <> 0 Then
		LogError("BytesToHex: Buffer length mod 4 not zero! Hex result will be incomplete!")
	End If
	Dim resultHexBuilder As StringBuilder
	resultHexBuilder.Initialize
	Dim val As Int
	For index = 0 To Buffer.Length - 1
		Select (index Mod 4)
			Case 0
				val = Bit.ShiftLeft(Bit.And(Buffer(index), 0xff), 24)
			Case 1
				val = Bit.Or(Bit.ShiftLeft(Bit.And(Buffer(index), 0xff), 16), val)
			Case 2
				val = Bit.Or(Bit.ShiftLeft(Bit.And(Buffer(index), 0xff), 8), val)
			Case 3
				val = Bit.Or(Bit.ShiftLeft(Bit.And(Buffer(index), 0xff), 0), val)
				resultHexBuilder.Append(Bit.ToHexString(val))
		End Select
	Next
	Return resultHexBuilder.ToString
End Sub

Public Sub CheckPassword (Password As String) As Int
	Dim hexedPasswordHash As String = BytesToHex(GenerateSHA1Hash(Password))
	Return CheckPasswordHash(hexedPasswordHash)
End Sub

Public Sub CheckPasswordAsync (Password As String, ID As String)
	Dim hexedPasswordHash As String = BytesToHex(GenerateSHA1Hash(Password))
	Sleep(0)
	CheckPasswordHashAsync(hexedPasswordHash, ID)
End Sub

Public Sub CheckPasswordHash (PasswordHash As String) As Int
	Dim resultCountPasswordSeen As Int = 0
	PasswordHash = PasswordHash.ToUpperCase
	Dim reader As JavaObject = GenerateReader(OpenDatabaseInputStream, internalCharSet, 30720)
	Try
		Do While True
			Dim readedLine As String = reader.RunMethod("readLine", Null)
			If readedLine = "null" Then
				Exit
			Else If readedLine.Contains(PasswordHash) Then
				Dim doubleDotPosition As Int = readedLine.LastIndexOf(":")
				readedLine = readedLine.SubString(doubleDotPosition + 1)
				resultCountPasswordSeen = readedLine
				Exit
			End If
		Loop
	Catch
		Log(LastException)
	End Try
	reader.RunMethod("close", Null)
	Return resultCountPasswordSeen
End Sub

Public Sub CheckPasswordHashAsync (PasswordHash As String, ID As String)
	PasswordHash = PasswordHash.ToUpperCase 'The hash must be uppercase. All password hashes in the database are uppercase.
	Dim databaseCount As Long = 0 'Readed bytes
	Dim databaseSize As Long = File.Size(internalDatabaseDirName, internalDatabaseFileName) 'Byte size of the database
	Dim countInStream As CountingInputStream = OpenDatabaseInputStream
	Dim reader As JavaObject = GenerateReader(countInStream, internalCharSet, 30720)
	Dim checkEventArgs As CheckPasswordEventArgs
	checkEventArgs.Initialize(PasswordHash, ID)
	RaiseCheckProgressEvent(checkEventArgs)
	Try
		Dim checkedPasswords As Long = 0
		Do While True
			Dim readedLine As String = reader.RunMethod("readLine", Null)
			If readedLine = "null" Then
				Exit
			Else If readedLine.Contains(PasswordHash) Then
				Dim colonPosition As Int = readedLine.LastIndexOf(":")
				readedLine = readedLine.SubString(colonPosition + 1)
				checkEventArgs.Progress = 1.0
				checkEventArgs.Result = readedLine
				Exit
			End If
			checkedPasswords = checkedPasswords + 1
			If databaseCount <> countInStream.Count Then
				databaseCount = countInStream.Count
				checkEventArgs.PasswordsChecked = checkedPasswords
				checkEventArgs.Progress = (databaseCount / databaseSize)
				RaiseCheckProgressEvent(checkEventArgs)
				Sleep(0)
				If checkEventArgs.Aborted Then
					Exit
				End If
			End If
		Loop
	Catch
		Log(LastException)
	End Try
	reader.RunMethod("close", Null)
	'RaiseCheckProgressEvent(checkEventArgs)
	RaiseCheckDoneEvent(checkEventArgs)
End Sub

'This will generate a SHA-1 hash of the given password.
Public Sub GenerateSHA1Hash (Password As String) As Byte()
	Dim messageDigest As JavaObject
	messageDigest.InitializeStatic("java.security.MessageDigest")
	messageDigest = messageDigest.RunMethodJO("getInstance", Array As Object("SHA-1"))
	Return messageDigest.RunMethod("digest", Array As Object(Password.GetBytes("UTF8")))
End Sub

Private Sub GenerateReader (InStream As InputStream, CharSet As String , BufferSize As Int) As JavaObject
	Dim inputStreamReader As JavaObject
	inputStreamReader.InitializeNewInstance("java.io.InputStreamReader", Array As Object(InStream, CharSet))
	Dim bufferedReader As JavaObject
	bufferedReader.InitializeNewInstance("java.io.BufferedReader", Array As Object(inputStreamReader, BufferSize))
	Return bufferedReader
End Sub

Public Sub Initialize (DirName As String, FileName As String, Owner As Object, EventName As String)
	internalDatabaseDirName = DirName
	internalDatabaseFileName = FileName
	internalEventName = EventName
	internalOwner = Owner
End Sub

Private Sub OpenDatabaseInputStream As CountingInputStream
	Dim resultCountingInStream As CountingInputStream
	resultCountingInStream.Initialize(File.OpenInput(internalDatabaseDirName, internalDatabaseFileName))
	Return resultCountingInStream
End Sub

Private Sub RaiseCheckDoneEvent (EventArgs As CheckPasswordEventArgs)
	Dim eventName As String = internalEventName & "_CheckDone"
	If internalOwner <> Null And SubExists(internalOwner, eventName) Then
		CallSubDelayed2(internalOwner, eventName, EventArgs)
	End If
End Sub

Private Sub RaiseCheckProgressEvent (EventArgs As CheckPasswordEventArgs)
	Dim eventName As String = internalEventName & "_CheckProgress"
	If internalOwner <> Null And SubExists(internalOwner, eventName) Then
		CallSubDelayed2(internalOwner, eventName, EventArgs)
	End If
End Sub
